/* apollo-client.h
 *
 * $Id: apollo-client.h,v 1.1.2.1 2002/06/27 14:52:34 amos Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

/*! \file apollo-client.h remote control client for apollo */

#ifndef APOLLO_CLIENT_H_INCLUDED
#define APOLLO_CLIENT_H_INCLUDED

#include "../glibtypes.h"
#ifdef __cplusplus
extern "C" {
#endif

gboolean setup_ctrlsocket(void);
void cleanup_ctrlsocket(void);
void start_ctrlsocket(void);
void check_ctrlsocket(void);
gint ctrlsocket_get_session_id(void);

enum
{
	CMD_GET_VERSION, CMD_PLAYLIST_ADD, CMD_PLAY, CMD_PAUSE, CMD_STOP,
	CMD_IS_PLAYING, CMD_IS_PAUSED, CMD_GET_PLAYLIST_POS,
	CMD_SET_PLAYLIST_POS, CMD_GET_PLAYLIST_LENGTH, CMD_PLAYLIST_CLEAR,
	CMD_GET_OUTPUT_TIME, CMD_JUMP_TO_TIME, CMD_GET_VOLUME,
	CMD_SET_VOLUME, CMD_GET_SKIN, CMD_SET_SKIN, CMD_GET_PLAYLIST_FILE,
	CMD_GET_PLAYLIST_TITLE, CMD_GET_PLAYLIST_TIME, CMD_GET_INFO,
	CMD_GET_EQ_DATA, CMD_SET_EQ_DATA, CMD_PL_WIN_TOGGLE,
	CMD_EQ_WIN_TOGGLE, CMD_SHOW_PREFS_BOX, CMD_TOGGLE_AOT,
	CMD_SHOW_ABOUT_BOX, CMD_EJECT, CMD_PLAYLIST_PREV, CMD_PLAYLIST_NEXT,
	CMD_PING, CMD_GET_BALANCE, CMD_TOGGLE_REPEAT, CMD_TOGGLE_SHUFFLE, CMD_MAIN_WIN_TOGGLE, CMD_PLAYLIST_ADD_URL_STRING,
	CMD_IS_EQ_WIN, CMD_IS_PL_WIN, CMD_IS_MAIN_WIN, CMD_PLAYLIST_DELETE,
	CMD_IS_REPEAT, CMD_IS_SHUFFLE,
	CMD_GET_EQ, CMD_GET_EQ_PREAMP, CMD_GET_EQ_BAND,
	CMD_SET_EQ, CMD_SET_EQ_PREAMP, CMD_SET_EQ_BAND,
	CMD_QUIT,

	CMD_PLAYFIRST_ADD = 1000
};

typedef struct
{
	guint16 version;
	guint16 command;
	guint32 data_length;
}
ClientPktHeader;

typedef struct
{
	guint16 version;
	guint32 data_length;
}
ServerPktHeader;

gint connect_to_session(gint session);	/* Do NOT use this! This is only for control socket initialization now. */
void remote_playlist(gint session, gchar ** list, gint num, gboolean enqueue, gboolean playfirstp);
gint remote_get_version(gint session);
void remote_playlist_add(gint session, GList * list);
void remote_playlist_delete(gint session, gint pos);
void remote_play(gint session);
void remote_pause(gint session);
void remote_stop(gint session);
gboolean remote_is_playing(gint session);
gboolean remote_is_paused(gint session);
gint remote_get_playlist_pos(gint session);
void remote_set_playlist_pos(gint session, gint pos);
gint remote_get_playlist_length(gint session);
void remote_playlist_clear(gint session);
gint remote_get_output_time(gint session);
void remote_jump_to_time(gint session, gint pos);
void remote_get_volume(gint session, gint * vl, gint * vr);
gint remote_get_main_volume(gint session);
gint remote_get_balance(gint session);
void remote_set_volume(gint session, gint vl, gint vr);
void remote_set_main_volume(gint session, gint v);
void remote_set_balance(gint session, gint b);
gchar *remote_get_skin(gint session);
void remote_set_skin(gint session, gchar * skinfile);
gchar *remote_get_playlist_file(gint session, gint pos);
gchar *remote_get_playlist_title(gint session, gint pos);
gint remote_get_playlist_time(gint session, gint pos);
void remote_get_info(gint session, gint * rate, gint * freq, gint * nch);
void remote_main_win_toggle(gint session, gboolean show);
void remote_pl_win_toggle(gint session, gboolean show);
void remote_eq_win_toggle(gint session, gboolean show);
gboolean remote_is_main_win(gint session);
gboolean remote_is_pl_win(gint session);
gboolean remote_is_eq_win(gint session);
void remote_show_prefs_box(gint session);
void remote_toggle_aot(gint session, gboolean ontop);
void remote_eject(gint session);
void remote_playlist_prev(gint session);
void remote_playlist_next(gint session);
void remote_playlist_add_url_string(gint session, gchar * string);
gboolean remote_is_running(gint session);
void remote_toggle_repeat(gint session);
void remote_toggle_shuffle(gint session);
gboolean remote_is_repeat(gint session);
gboolean remote_is_shuffle(gint session);
void remote_get_eq(gint session, gfloat *preamp, gfloat **bands);
gfloat remote_get_eq_preamp(gint session);
gfloat remote_get_eq_band(gint session, gint band);
void remote_set_eq(gint session, gfloat preamp, gfloat *bands);
void remote_set_eq_preamp(gint session, gfloat preamp);
void remote_set_eq_band(gint session, gint band, gfloat value);
void remote_quit(gint session);

#ifdef __cplusplus
};
#endif

#endif // APOLLO_CLIENT_H_INCLUDED
