/* qmautopointer.cpp
 *
 * $Id: qmautopointer.cpp,v 1.3 2002/03/07 03:44:16 mariuss Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

/*! \file qmautopointer.cpp
  Definition of QmAutoPointer class.
*/

/*!
  \class QmAutoPointer qmautopointer.hpp
  \brief A template that makes dynamic objects exist in a given scope.

  \warning Don't try to use the address of an object that allocated on the stack, WILL result in a crash.

  It keeps a pointer to a dynamicly allocated object, will delete the object as soon as this object is deleted,
  that means that the dynamic object will only exists in the scope of this object.

  Just make a variable of this template on the stack and use it instead of the pointer.
  It will check for a null pointer and print a debug message.

  Example:
  \code

  void main()
  {
      QmAutoPointer<QmConfig> confptr( new QmConfig() );

      qDebug( confptr->configPath() ); // You can use it as it was a pointer to the real object.

      QmAutoPointer<QmConfig> tmpptr = confptr; // Change ownership
      
      tmpptr.reset(); // Deletes the pointer

  } // The destructor of the eZAutoPointer template is called and the document is deleted.
  \endcode

  The auto pointer can also be used as a member of a class, this ensures that the pointer
  is deleted when the class is deleted.

  The code is based on the auto_ptr template which is a part of the draft C++ standard library [ANSI95].
  A short discussion of this can be found in Pattern Languages of Program Design 2, Chapter 1 Managing Dynamic Objects in C++, part 2.

  This class is only created to enable auto pointers on platforms which doesn't have one available.
*/
