/* qmbrowseritem.cpp
 *
 * $Id: qmbrowseritem.cpp,v 1.9 2002/03/07 03:44:16 mariuss Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmbrowseritem.h"

#include "qmconfig.h"
#include "qmlistviewitem.h"

#include <qlistview.h>
#include <qtextstream.h>

/**
 * \file qmbrowseritem.cpp
 * \brief Generic item in the browser, parent of the playlist/directory browser items
 */
/*!
  \class QmBrowserItem qmbrowseritem.h
  \brief Represents an item in the browser.

  This class is meant as a base class for items in the browser, i.e.
  the playlist/directory browser.
*/


/*!
  Constructs a root item.  The string \a text will be displayed
  in the zero'th column and \a filename will be file corresponding to this item.
  The filename is retrievable with fileName().
*/

QmBrowserItem::QmBrowserItem(
    QListView *parent,
    const QString &text,
    const QString &filename)
    : QmListViewItem (parent),
      m_FileName(filename)
{
    setText(0, text);
}


/*!
  Constructs a root item after \a after.  The string \a text will be displayed
  in the zero'th column and \a filename will be file corresponding to this item.
  The filename is retrievable with fileName().
*/

QmBrowserItem::QmBrowserItem(
	QListView *parent,
	QListViewItem *after,
	const QString &text,
	const QString &filename)
	: QmListViewItem(parent, after),
	  m_FileName(filename)
{
	setText(0, text);
}


/*!
  Constructs a child of \a parent after \a after.  The string \a text will be displayed
  in the zero'th column and \a filename will be file corresponding to this item.
  The filename is retrievable with fileName().
*/
	
QmBrowserItem::QmBrowserItem(
	QmBrowserItem *parent,
	QListViewItem *after,
	const QString &text,
	const QString &filename)
	: QmListViewItem(parent, after),
	  m_FileName(filename)
{
	setText(0, text);
}


/*!

*/

QmBrowserItem::~QmBrowserItem()
{
}


/*!
  Retrieve the filename as given in the constructor.
  
  \return The filename.
*/

const QString&
QmBrowserItem::fileName() const
{
	return m_FileName;
}


/*!
  Sets the filename to \a filename.
*/

void
QmBrowserItem::setFileName(
	const QString &filename)
{
	m_FileName = filename;
}

