/* ;-*-c++-*-
 * 
 * qmconfigpage.h
 *
 * $Id: qmconfigpage.h,v 1.9 2002/03/07 03:44:16 mariuss Exp $
 * 
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef QMCONFIGPAGE_H_INCLUDED
#define QMCONFIGPAGE_H_INCLUDED

#include <qwidget.h>
#include "qmconfigdialog.h"

/* \file qmconfigpage.h
  Definition of QmConfigPage class which is the base class for all configuration pages.
*/

class QmConfigPage : public QWidget
{
	Q_OBJECT
public:
	QmConfigPage(QmConfigDialog *parent = 0, const char *name = 0, WFlags f = 0);
	virtual ~QmConfigPage();

	/*!
	  Use this function initialize the configuration page with the values
	  last saved.  If no values have been saved, use default values.
	  
	  There is typically no need for subclasses to call this function.
	  The configuration dialog, QmConfigDialog, will call this function
	  when the page is being displayed.

	  \sa save()
	 */
	
	virtual void load() = 0;

	/*!
	  Use this function to save the variables in this configuration page.
	  
	  There is typically no need for subclasses to call this function.
	  The configuration dialog, QmConfigDialog, will iterate through all
	  its configuration pages and call this function for each of them.

	  If you write a subclass \e not to be used in the configuration dialog,
	  however, you should call this when you close the window.  QmPlayListPropertyPage
	  does this.

	  \sa load()
	*/
	
	virtual void save() = 0;

	virtual void discard();

public slots: 
	virtual void changed();


private:
	QmConfigDialog *m_pParent;
};

#endif // QMCONFIGPAGE_H_INCLUDED
