/* qmdropindicator.cpp
 *
 * $Id: qmdropindicator.cpp,v 1.7 2002/03/30 11:49:06 kyllingstad Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmdropindicator.h"

#include <qpainter.h>
#include <qpen.h>

/**
 * @file qmdropindicator.cpp
 * @brief Widget showing drop-point in the playlist during dragging
 */

/*!
  \class QmDropIndicator qmdropindicator.h
  \brief Represents the line used in a list view during drag'n drop.

  This widget is a simple line.  It's used to indicate exactly where
  a dragged item during a drag'n drop operation will be inserted.  The
  line is placed between the two items the dragged item(s) will be placed.

  \sa QmListView
*/


/*!
  Initializes the drop indicator with a height of 2 
*/

QmDropIndicator::QmDropIndicator(
	QWidget *parent,
	const char *name)
	: QWidget(parent, name)
{
//	setFixedHeight(6);
	setFixedHeight(2);
}


/*!
  Destroys the drop indicator.
*/

QmDropIndicator::~QmDropIndicator()
{
}


/*!
  Custom paintevent. Draws a simple line.
*/

void
QmDropIndicator::paintEvent(
	QPaintEvent *)
{
	QPainter p(this);
	p.setPen(black);

	p.drawLine(0, 0, width(), 0);
	p.drawLine(0, 1, width(), 1);
	
	// The commented code below draws a nice little "triangle"
	// at each end of the widget which is custom to do for
	// drop indicators like this.  The problem is, though, that
	// this widget needs to be transparent in that case.
	
	/*
	p.drawLine(0, 2, width(), 2);
	p.drawLine(0, 3, width(), 3);

	p.drawLine(0, 0, 0, 5);
	p.drawLine(1, 1, 1, 4);

	p.drawLine(width() - 1, 0, width() - 1, 5);
	p.drawLine(width() - 2, 1, width() - 2, 4);
	*/
}

