/* qmnullinfo.cpp
 *
 * $Id: qmnullinfo.cpp,v 1.3 2002/03/07 03:44:17 mariuss Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmnullinfo.h"

#include <qfileinfo.h>

/*!
  \class QmNullInfo qmnullinfo.h
  \brief A dummy songinfo implemention.

  This is a class null object. Its purpose is for the rest of the code to
  avoid having to check if a songinfo object pointer is null.

  The only information this class will provide is the filename. See
  get(Field) for more information.
*/


/*!
  Constructs a null object based on \a filename.
*/

QmNullInfo::QmNullInfo(
	const QString &filename)
{
	QFileInfo fi(filename);
	m_Filename = fi.fileName();
}


/*!

*/

QmNullInfo::~QmNullInfo()
{
}


/*!
  \return The filename if \a f is \p Title, otherwise a blank string is returned.
 */

const QString&
QmNullInfo::get(
	Field f) const
{
	if(f == Title)
		return m_Filename;
	else
		return blank();
}
