/* qmpropertydialog.cpp
 *
 * $Id: qmpropertydialog.cpp,v 1.2 2002/03/07 03:44:17 mariuss Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmpropertydialog.h"
#include "qmconfigpage.h"

#include <qlayout.h>
#include <qpushbutton.h>
#include <qtabwidget.h>

/*!
  \class QmPropertyDialog qmpropertydialog.h
  \brief A placeholder for QmConfigPage pages.

  This is a dialog for QmConfigPage pages. Typically used in a property
  dialog box scenario.

  This class is not meant to be derived from, at least not for providing
  different property dialog boxes. Instead, simply create an object of
  this class and objects of the various pages you want. Then call
  addPage(const QString&, QmConfigPage*).
*/


/*!
  Constructs an empty property dialog.
*/

QmPropertyDialog::QmPropertyDialog(
	QWidget *parent,
	const char *name)
	: QDialog(parent, name)
{
	setCaption(tr("Apollo - Properties"));

	QVBoxLayout *main = new QVBoxLayout(this, 2);
	QHBoxLayout *bottom = new QHBoxLayout;

	m_pClose = new QPushButton(tr("&Close"), this);
	m_pClose->setFixedWidth(m_pClose->sizeHint().width());
	m_pClose->setDefault(true);
	connect(m_pClose, SIGNAL(clicked()), this, SLOT(closeClicked()));

	bottom->addStretch();
	bottom->addWidget(m_pClose);
	
	m_pPages = new QTabWidget(this);

	main->addWidget(m_pPages, 2);
	main->addLayout(bottom);
}


/*!

*/

QmPropertyDialog::~QmPropertyDialog()
{
}


/*!
  Call this to add another page with title \a name. The page widget \a page will
  be properly reparented.
 */

void
QmPropertyDialog::addPage(
	const QString& name,
	QmConfigPage *page)
{
	CHECK_PTR(page);
	page->reparent(m_pPages, 0, QPoint(0, 0));
	m_pPages->insertTab(page, name);
}



/*!

 */

void
QmPropertyDialog::closeClicked()
{
	accept();
}
