/* ;-*-c++-*-
 * 
 * qmshortcutpage.h
 *
 * $Id: qmshortcutpage.h,v 1.13 2002/03/21 14:31:28 amos Exp $
 * 
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef QMSHORTCUTPAGE_H_INCLUDED
#define QMSHORTCUTPAGE_H_INCLUDED

#include "qmconfigpage.h"

#include <qmap.h>

class QCheckBox;
class QLabel;
class QListView;
class QListViewItem;
class QPushButton;
class QRadioButton;
class QTimer;
class QWidget;
class QmAccelManager;
class QHButtonGroup;
class Group;

/*! \file qmshortcutpage.h
  Definition of QmShortcutPage class which is the page for editing shortcut keys.
*/

class QmShortcutPage : public QmConfigPage
{
	Q_OBJECT
public:
	QmShortcutPage(QmConfigDialog *parent = 0, const char *name = 0);
	virtual ~QmShortcutPage();

	virtual void load();
	virtual void save();
	virtual void discard();

protected:
	virtual void keyPressEvent(QKeyEvent *e);

private slots:
	void grabShortcut();
	void selectionChanged();
	void removeKey();
    void defaultKey();
    void timeoutRelease();
    void keyTypeChange( int );
    void grabShortcut( QListViewItem * );
    void grabDelayedShortcut( QListViewItem * );
    void delayTimeout();

private:
    QString		   keyName(QKeyEvent *e) const;
    QString        commandForItem( QListViewItem * ) const;
    QListViewItem *itemForCommand( const QString & ) const;
    void           addKeys( const Group *g, QListViewItem *parent );
    void           checkKey( int key );
	void           removeKey( QListViewItem * );
    bool           askAboutKeyBound( int key, const QString &command, const QString &keyName );

	QLabel		   *m_pDesc;
	QPushButton    *m_pGrab;
	QCheckBox	   *m_pShift;
	QCheckBox	   *m_pCtrl;
	QCheckBox	   *m_pAlt;
    QRadioButton   *m_pNoKey;
    QRadioButton   *m_pDefKey;
    QRadioButton   *m_pCustKey;
    QHButtonGroup  *m_pKeyGroup;
	QListView	   *m_pKeyList;
    QMap<QListViewItem *, QString> m_ItemCommandMap;
    QMap<QString, QListViewItem *> m_CommandItemMap;
    QTimer		   *m_pTimer, *m_pDelayTimer;
	QmAccelManager *m_pAm;
};

#endif // QMSHORTCUTPAGE_H_INCLUDED
