/* qmsonginfo.cpp
 *
 * $Id: qmsonginfo.cpp,v 1.7.2.1 2002/09/23 19:26:51 kyllingstad Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmsonginfo.h"

#include "qmmp3info.h"
#include "qmnullinfo.h"

#include <qfileinfo.h>

#include <iostream>

/*!
  \class QmSongInfo qmsonginfo.h
  \brief Provides information on a song.

  This is the interface from which the user interface retrieves
  information about the songs from. Derive from this class to deal
  with the various song formats, such as MP3, Ogg etc.

  To get a QmSongInfo object, call create(const QString&).
*/

/*!
  \enum QmSongInfo::Field
  The field values that can be retrieved.
*/
	

/*!

*/

QmSongInfo::QmSongInfo()
	: m_Blank("")
{
}


/*!

*/

QmSongInfo::~QmSongInfo()
{
}


/*!
  Inspects \a filename and returns an appropriate songinfo object. This function
  will never return null, regardless of the file is understood or not. If the
  file is of an unknown type, a dummy songinfo object will be returned.

  \warning The caller is responsible for freeing the object returned.
 */

QmSongInfo*
QmSongInfo::create(
	const QString& filename)
{
	QFileInfo fi(filename);

	if(fi.extension(false).lower() == "mp3")
#ifdef ENABLE_ID3
    {
		return new QmMp3Info(filename);
    }
#else
		return new QmNullInfo(filename);
#endif	
//	else if(fi.extension(false).lower() == "ogg")
//		return new QmOggInfo(filename);
	else
		return new QmNullInfo(filename);
}


/*!
  Provided for convenience to subclasses so that every subclass does not have
  to have a blank string among its private variables.

  \return A blank string.
*/

const QString &
QmSongInfo::blank() const
{
	return m_Blank;
}

