/* ;-*-c++-*-
 * qmsongitem.h
 *
 * $Id: qmsongitem.h,v 1.32 2002/03/31 23:52:42 kyllingstad Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef QMSONGITEM_H_INCLUDED
#define QMSONGITEM_H_INCLUDED

#include <qfileinfo.h>

#include "qmplaylistitem.h"
#include "qmplaylist.h"

/*! \file qmsongitem.h
  Declaration of QmSongItem class which represents a song in the playlist.
*/

class QStringList;
class QTextStream;
class QmSongInfo;

class QmSongItem : public QmPlayListItem
{
    friend class QmPlayList;
	
public:
    QmSongItem(QListView* parent, QListViewItem* after, QString filename,
			   long lengthSeconds=0, int bitrate=0, bool vbr=false, QString title="",
			   QString artist="", QString album="", QString cdPosition="", bool multiArtist=false,
			   int year=0, QString comment="", QString displayString="");
    QmSongItem(QmPlayListItem* parent, QListViewItem* after, QString filename,
			   long lengthSeconds=0, int bitrate=0, bool vbr=false, QString title="",
			   QString artist="", QString album="", QString cdPosition="", bool multiArtist=false,
			   int year=0, QString comment="", QString displayString="");
    virtual ~QmSongItem();

    void				setSongTitle(const QString &title);
    void				setArtist(const QString &artist);
    void				setAlbum(const QString &album);
	void                setMultiArtist(bool ma);
	void 				setMark(bool mark);
    void                setBitrate(int bitrate);
	void                setRating(int rating);
	int                 incRating();
	int                 decRating();
	int                 played() const;
	void                resetPlayed();
    void                setVariableBitrate(bool vbr);
    virtual void        setDisplayFormat(const QString& format, const QString& multiFormat);
    virtual void        setDisplayFormat(const char *format, const char *multiFormat);
    virtual void        setDisplayString(const QString &display);
    virtual QString     displayString() const;

    virtual bool        isBad() const;

    virtual void 		setPlaying(bool p);
    virtual QString		filePath() const;
    virtual QString     text(int) const;
	virtual QString		save( const QString &rel_path ) const;
	virtual bool		isSong() const;
    virtual void 		paintCell(QPainter *p, const QColorGroup &cg, int col, int w, int a);
    virtual void 		paintFocus(QPainter *p, const QColorGroup &cg, const QRect &r);
    virtual const QString& title() const { return m_Title; }
    virtual const QString& artist() const { return m_Artist; }
    virtual const QString& album() const { return m_Album; }
    virtual const QString& cdPosition()  const { return m_CdPosition; }
    virtual int         bitrate() const { return m_Bitrate; }
	virtual int         rating() const { return m_Rating; }
    virtual bool        variableBitrate() const { return m_VariableBitrate; }
	virtual bool        multiArtistp() const { return m_MultiArtistAlbum; }
             
    virtual void        setup();
    virtual void        readInfo(const char *format, const char *multiFormat);
    virtual void        writeXml(QTextStream &out) const;

	QmSongInfo*			info() const;

private:
    void                init(long lengthSeconds = 0);
    inline void         setField(QString &field, const QString &value, bool &changed);

    QFileInfo			m_File;
    QString     		m_Title;
    QString             m_Artist;
    QString             m_Album;
    QString             m_CdPosition;
	QString             m_Comment;
    QString             m_DisplayString;
	int                 m_Year;
    int                 m_Bitrate;
	int                 m_Rating;
	int                 m_Played;
    bool                m_VariableBitrate;
    bool				m_Playing;
	bool				m_IsBad;
	bool                m_MultiArtistAlbum;
};

#endif // QMSONGITEM_H_INCLUDED
