/* qmxmpplayer.cpp
 *
 * $Id: qmxmpplayer.cpp,v 1.2 2002/03/07 03:44:17 mariuss Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000-2002 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifdef ENABLE_XMP

#include "qmxmpplayer.h"

extern "C" {
#include "xmp.h"
}

#include <stdio.h>

/*!
  \class QmXmpPlayer qmxmpplayer.h
  \brief Provides functionality for playing tracker modules.

  This class is a wrapper around XMP (Extended Module Player) which makes it
  possible to play tracker modules. XMP 2.0.4 can play 48 formats:

  XM (Fast Tracker II), MOD (Noise/Fast/Protracker), MOD (Startrekker/Audio Sculpure),
  M15 (Soundtracker), IT (Impulse Tracker), S3M (Scream Tracker 3),
  STM (Scream Tracker 2), STX (STMIK 0.2), MTM (Multitracker), MTN (Soundtracker 2.6/Ice Tracker),
  IMF (Imago Orpheus), PTM (Protracker), MDL (Digitrakker), ULT (Ultra Tracker),
  MED (MED/OctaMED), PTM (Poly Tracker), DIGI (DIGI Booster), EMOD (Quadra Composer),
  OKT (Oktalyzer), SFX (SoundFX 1.3), FAR (Farandole Composer), STIM (Slamtilt),
  FC-M (FC-M Packer), KSM (Kefrens Sound Machine), WN (Wanton Packer), PM (Power Music),
  KRIS (ChipTracker), UNIC (Unic Tracker), PRU1 (ProRunner 1.0), PRU2 (ProRunner 2.0),
  PM01 (Promizer 0.1), PM10 (Promizer 1.0c), PM18 (Promizer 1.8a), PM20 (Promizer 2.0),
  PM40 (Promizer 4.0), AC1D (AC1D Packer), PP10 (Pha Packer), XANN (XANN Packer),
  ZEN (Zen Packer), NP (NoisePacker), DI (Digital Illusions), MP (Module Protector),
  669 (Composer 669), FNK (Funktracker), AMD (Amusic Adlib Tracker),
  RAD (Reality Adlib Tracker), CRB (Heatseeker), ALM (Aley Keptr)  

  This code is based on the simple.c example that comes with the library.

  http://xmp.helllabs.org
  http://sourceforge.net/projects/xmp/
*/

struct xmp_control QmXmpPlayer::m_Control;
struct xmp_module_info QmXmpPlayer::m_Info;

/*!

*/

QmXmpPlayer::QmXmpPlayer()
{
	m_Filename[0] = '\0';
	
	setbuf(stdout, 0);	
//	xmp_init(argc, argv, &ctl);
	xmp_init(0, 0, &m_Control);
	m_Control.verbose = 0;

	if(xmp_open_audio(&m_Control) < 0)
	{
		printf("Cant open audio device.\n");
	}

//	xmp_register_event_callback(QmXmpPlayer::process);
}


/*!

*/

QmXmpPlayer::~QmXmpPlayer()
{
	xmp_close_audio();
}


/*!

 */

bool
QmXmpPlayer::isPlaying() const
{
	return false;
}


/*!

 */

bool
QmXmpPlayer::isPaused() const
{
	return false;
}


/*!

 */

void
QmXmpPlayer::play(
	const QString &Filename)
{
	for(unsigned int i = 0; i < Filename.length(); i++)
		m_Filename[i] = Filename[i];
	
	play();
}


/*!

 */

void
QmXmpPlayer::play()
{	
	if(xmp_load_module(m_Filename) < 0)
	{
		printf("Error loading module %s\n", m_Filename);
		return;
	}

	xmp_get_module_info(&m_Info);
	printf("%s (%s)\n", m_Info.name, m_Info.type);
	xmp_play_module();
	printf("\n");
}


/*!

 */

void
QmXmpPlayer::stop()
{
}


/*!

 */

void
QmXmpPlayer::pause()
{
}


/*!

 */

void
QmXmpPlayer::restart()
{
}


/*!

 */

void
QmXmpPlayer::jump(
	int)
{
}


/*!

 */

void
QmXmpPlayer::process(
	unsigned long i)
{
	unsigned long msg = i >> 4;
    static int oldpos = -1;
    static int pos, pat;
 
    switch (i & 0xf)
	{
    case XMP_ECHO_ORD:
        pos = msg & 0xff;
        pat = msg >> 8;
        break;
		
    case XMP_ECHO_ROW:
        if (!(msg & 0xff) || pos != oldpos)
		{
            printf ("\rOrder %02X/%02X - Pattern %02X/%02X - Row      ",
					pos, m_Info.len, pat, m_Info.pat - 1);
            oldpos = pos;
        }
        printf ("\b\b\b\b\b%02X/%02X", (int)(msg & 0xff), (int)(msg >> 8));
        break;
    }
}

#endif // ENABLE_XMP
